import argparse
import os
import json_leases_to_unbound
from argparse import ArgumentParser


def main_cli():
    parser = ArgumentParser()
    parser.add_argument('--log-level', help='set the logging level', default='INFO', choices=['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'])
    parser.add_argument('--source', help='source leases directory', default='/run/slaac-resolver/')
    parser.add_argument('--domain', help='default domain to use', default='lan')
    parser.add_argument('--unbound-server', help='unbound server to connect to (IP:port)', default=None)
    parser.add_argument('--config-file', help='unbound config file path', default=None)

    inputargs = parser.parse_args()
    json_leases_to_unbound.main(
        log_level=inputargs.log_level,
        source=inputargs.source,
        domain=inputargs.domain,
        unbound_server=inputargs.unbound_server,
        config_file=inputargs.config_file
    )


import os
import logging
import json
import time
import sys
import subprocess
import shutil
from argparse import ArgumentParser
import ipaddress
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler

DEFAULT_DOMAIN = 'lan'
UNBOUND_CONTROL_PATH = None
active_leases = {}

class DirChangeHandler(FileSystemEventHandler):
    def __init__(self, source_dir):
        self.source_dir = source_dir
        logging.debug(f"DirChangeHandler initialized with source: {source_dir}")

    def on_any_event(self, event):
        global active_leases, default_domain
        """Handle any file system event."""
        logging.debug(f"Event detected: {event.event_type} on {event.src_path}")
        if event.event_type  in ("created", "modified") and not event.is_directory:
            try:
                logging.debug(f"Process leases from file: {event.src_path}")
                process_lease_file(event.src_path)
            except Exception as e:
                logging.error(f"Error reading file {event.src_path}: {e}")
        if event.event_type == "deleted":
            try:
                logging.debug(f"Lease file {event.src_path} has been deleted")
                delete_leases_from_file(event.src_path)
            except Exception as e:
                logging.error(f"Error deleting entries from file {event.src_path}")

logger = logging.getLogger(__name__)

def find_unbound_control():
    """Find the unbound-control binary in common locations"""
    global UNBOUND_CONTROL_PATH
    
    if UNBOUND_CONTROL_PATH:
        return UNBOUND_CONTROL_PATH
    
    # Common paths where unbound-control might be located
    common_paths = [
        '/usr/sbin/unbound-control',
        '/sbin/unbound-control', 
        '/usr/local/sbin/unbound-control',
        '/usr/bin/unbound-control',
        '/bin/unbound-control'
    ]
    
    # First try using 'which' command
    try:
        UNBOUND_CONTROL_PATH = shutil.which('unbound-control')
        if UNBOUND_CONTROL_PATH:
            logger.debug(f"Found unbound-control using which: {UNBOUND_CONTROL_PATH}")
            return UNBOUND_CONTROL_PATH
    except Exception as e:
        logger.debug(f"Error using which: {e}")
    
    # Fallback to checking common paths
    for path in common_paths:
        if os.path.isfile(path) and os.access(path, os.X_OK):
            UNBOUND_CONTROL_PATH = path
            logger.debug(f"Found unbound-control at: {UNBOUND_CONTROL_PATH}")
            return UNBOUND_CONTROL_PATH
    
    raise FileNotFoundError("unbound-control binary not found in common locations")

def unbound_control(commands, input=None, server=None, config_file=None):
    """Execute unbound-control command"""
    try:
        unbound_control_path = find_unbound_control()
    except FileNotFoundError as e:
        logger.error(f"Cannot find unbound-control: {e}")
        return
    
    # Build command with optional server specification and config file
    cmd = [unbound_control_path]
    
    # Add config file if provided
    if config_file and os.path.exists(config_file):
        cmd.extend(['-c', config_file])
    
    # Add server specification (IP:port)
    if server:
        cmd.extend(['-s', server])
    
    cmd.extend(commands)

    input_string = None
    if input:
        input_string = '\n'.join(input) + '\n'
    logger.debug(f"Executing unbound-control command: {' '.join(cmd)}")
    result = subprocess.run(cmd, input=input_string, text=True, capture_output=True)
    logger.debug(f"unbound-control output: {result.stdout}")
    if result.stderr:
        logger.error(f"unbound-control error: {result.stderr}")
    
    return result

def apply_unbound_changes(dhcpd_changed, remove_rr, add_rr, server=None, config_file=None):
    """Apply changes to Unbound DNS."""
    if dhcpd_changed:
        if remove_rr:
            logger.info(f"Removing {len(remove_rr)} resource records")
            unbound_control(['local_datas_remove'], input=remove_rr, server=server, config_file=config_file)
        if add_rr:
            logger.info(f"Adding {len(add_rr)} resource records")
            unbound_control(['local_datas'], input=add_rr, server=server, config_file=config_file)

def add_to_unbound(hostname, address) -> list:
    global default_domain
    fqdn = f"{hostname}.{default_domain}"
    record_type = "A" if address.version == 4 else "AAAA"
    add_rr = [
        f"{address.reverse_pointer} PTR {fqdn}",
        f"{fqdn} IN {record_type} {address}"
    ]
    return add_rr

def remove_from_unbound(hostname, address) -> list:
    global default_domain
    fqdn = f"{hostname}.{default_domain}"
    record_type = "A" if address.version == 4 else "AAAA"
    remove_rr = [ 
        f"{address.reverse_pointer} PTR {fqdn}",
        f"{fqdn} IN {record_type} {address}"
    ]
    return remove_rr

def modify_on_unbound(hostname, address, prev_address) -> tuple:
    remove_rr = remove_from_unbound(hostname,prev_address)
    add_rr = add_to_unbound(hostname,address)
    return add_rr, remove_rr

def extract_leases(data) -> list:
    """Extract leases from json data."""
    leases = []
    for item in data:
        try:
            if item['AddressType'] == 'IPv4':
                address = '.'.join(map(str, item['Address']))
            elif item['AddressType'] == 'IPv6':
                address = ':'.join(map(str, item['Address']))
            else:
                logging.warning(f"Unknown AddressType {item['AddressType']} in lease data")
                continue
            leases.append({
                "type"    : item['AddressType'],
                "address" : ipaddress.ip_address(address),
                "hostname": item.get('Hostname', ''),
                "expire"  : item.get('Expire', ''),
            })
        except KeyError as e:
            logging.error(f"Missing expected key in lease data: {e}")
    return leases

def filter_leases_by_expiration(leases) -> list:
    filtered_leases = {}
    for lease in leases:
        if filtered_leases.get(lease['hostname']) is None:
            filtered_leases[lease['hostname']] = lease
        else:
            if filtered_leases[lease['hostname']]['expire'] < lease['expire']:
                filtered_leases[lease['hostname']] = lease
    return list(filtered_leases.values())

def read_file(file_path) -> list:
    """Read the leases from JSON file."""
    leases = []
    try:
        logging.debug(f"Reading file: {file_path}")
        with open(file_path, 'r') as file:
            data = json.load(file)
            leases.extend(extract_leases(data))
        logging.debug(f"File read successfully: {file_path}")
    except (FileNotFoundError, json.JSONDecodeError) as e:
        logging.error(f"Error reading file {file_path}: {e}")
    filtered_leases = filter_leases_by_expiration(leases)
    return filtered_leases

def delete_leases_from_file(file_path, server=None, config_file=None):
    global active_leases
    remove_rr = []
    filename = os.path.basename(file_path)
    logging.debug(f"Removing inactive lease file: {filename}")
    leases_to_remove = list(active_leases[filename].keys())
    for hostname in leases_to_remove:
        logging.info(f"Removing lease of hostname: {hostname} from deleted file: {filename}")
        remove_rr.extend(remove_from_unbound(hostname,active_leases[filename][hostname]))
    dhcp_changed = len(remove_rr) > 0
    apply_unbound_changes(dhcp_changed, remove_rr, [], server=server, config_file=config_file)
    del active_leases[filename]

def _remove_inactive_leases(leases,filename):
    """Remove leases that are no longer present in the source directory."""
    global active_leases
    remove_rr = []
    active_lease_entries = list(active_leases[filename].keys())
    for hostname in active_lease_entries:
        if hostname not in [lease['hostname'] for lease in leases]:
            logging.info(f"Removing inactive lease for hostname: {hostname} from file: {filename}")
            remove_rr.extend(remove_from_unbound(hostname,active_leases[filename][hostname]))
            del active_leases[filename][hostname]
    return remove_rr

def _process_lease_entries(lease_list, filename) -> tuple:
    """Process lease entries and update active leases."""
    global active_leases
    add_rr = []
    remove_rr = []
    for lease in lease_list:
        hostname = lease['hostname']
        address = lease['address']
        prev_address = active_leases[filename].get(hostname)
        if prev_address is None:
            active_leases[filename][hostname] = address
            add_rr.extend(add_to_unbound(hostname,address))
            logging.info(f"New lease added: {hostname} with address {address}")
        elif prev_address != address:
            logging.info(f"Lease for hostname {hostname} has changed from {prev_address} to {address}")
            active_leases[filename][hostname] = address
            add, rem = modify_on_unbound(hostname,address,prev_address)
            add_rr.extend(add)
            remove_rr.extend(rem)
        else:
            logging.debug(f"No change in lease for hostname {hostname} on file {filename}")
    remove_rr.extend(_remove_inactive_leases(lease_list,filename))
    logging.debug(f"Finished processing file: {filename}")
    return add_rr, remove_rr

def process_lease_file(file_path, server=None, config_file=None):
    """Process a single lease file."""
    global active_leases
    logging.info(f"Processing lease file: {file_path}")
    leases = read_file(file_path)
    if leases:
        filename = os.path.basename(file_path)
        active_leases.setdefault(filename, {})
        if not active_leases[filename]:
            logging.info(f"New lease file detected: {filename}")
        add_rr, remove_rr = _process_lease_entries(leases, filename)
        dhcp_changed = len(add_rr) > 0 or len(remove_rr) > 0
        apply_unbound_changes(dhcp_changed, remove_rr, add_rr, server=server, config_file=config_file)
    else:
        logging.warning(f"No valid leases found in file: {file_path}")
    return active_leases

def initial_run(source, server=None, config_file=None):
    """Initial run to process all lease files in the source directory."""
    global active_leases
    logging.info(f"Initial run on source file or directory: {source}")
    if not os.path.exists(source):
        logging.error(f"Source file or directory does not exist: {source}")
        sys.exit(1)
    if os.path.isfile(source):
        logging.info(f"Processing single file: {source}")
        process_lease_file(source, server=server, config_file=config_file)
        return
    if os.path.isdir(source):
        for filename in os.listdir(source):
            process_lease_file(os.path.join(source,filename), server=server, config_file=config_file)


def main(log_level: str, source: str, domain: str, unbound_server: str = None, config_file: str = None):
    level = getattr(logging, log_level.upper(), logging.INFO)
    logging.basicConfig(level=level, format='%(asctime)s - %(levelname)s - %(message)s')
    
    logging.info(f"Starting Json leases to Unbound with source: {source}")
    logging.info(f"Configuration: server={unbound_server}, config_file={config_file}")

    global default_domain
    default_domain = domain

    initial_run(source, server=unbound_server, config_file=config_file)

    event_handler = DirChangeHandler(source)
    observer = Observer()
    observer.schedule(event_handler, path=source, recursive=False)
    observer.start()
    try:
        while True:
            time.sleep(5)
    except KeyboardInterrupt:
        observer.stop()
    observer.join()


import pytest
import json
import ipaddress
import os
from unittest.mock import Mock, patch, MagicMock
from json_leases_to_unbound import core


class TestLeaseExtraction:
    """Tests for lease data extraction and filtering."""
    
    def test_extract_leases_valid_data(self):
        """Test extracting leases from valid JSON data."""
        data = [
            {
                "Address": [0x2001, 0xdb8, 0, 0, 0, 0, 0, 1],
                "AddressType": "IPv6",
                "Hostname": "test-host",
                "Expire": "2025-12-31T23:59:59Z"
            }
        ]
        
        leases = core.extract_leases(data)
        
        assert len(leases) == 1
        assert leases[0]["hostname"] == "test-host"
        # Note: hex 0x2001 = 8193, 0xdb8 = 3512 in decimal
        assert leases[0]["address"] == ipaddress.ip_address("8193:3512::1")
        assert leases[0]["expire"] == "2025-12-31T23:59:59Z"
    
    def test_extract_leases_missing_hostname(self):
        """Test extracting leases when hostname is missing."""
        data = [
            {
                "Address": [0x2001, 0xdb8, 0, 0, 0, 0, 0, 1],
                "AddressType": "IPv6",
                "Expire": "2025-12-31T23:59:59Z"
            }
        ]
        
        leases = core.extract_leases(data)
        
        assert len(leases) == 1
        assert leases[0]["hostname"] == ""
    
    def test_filter_leases_by_expiration_keeps_latest(self):
        """Test that filtering keeps the lease with the latest expiration."""
        leases = [
            {
                "hostname": "test-host",
                "address": ipaddress.ip_address("2001:db8::1"),
                "expire": "2025-12-30T23:59:59Z"
            },
            {
                "hostname": "test-host",
                "address": ipaddress.ip_address("2001:db8::2"),
                "expire": "2025-12-31T23:59:59Z"
            }
        ]
        
        filtered = core.filter_leases_by_expiration(leases)
        
        assert len(filtered) == 1
        assert filtered[0]["address"] == ipaddress.ip_address("2001:db8::2")
        assert filtered[0]["expire"] == "2025-12-31T23:59:59Z"
    
    def test_filter_leases_by_expiration_multiple_hostnames(self):
        """Test filtering with multiple different hostnames."""
        leases = [
            {
                "hostname": "host1",
                "address": ipaddress.ip_address("2001:db8::1"),
                "expire": "2025-12-31T23:59:59Z"
            },
            {
                "hostname": "host2",
                "address": ipaddress.ip_address("2001:db8::2"),
                "expire": "2025-12-31T23:59:59Z"
            }
        ]
        
        filtered = core.filter_leases_by_expiration(leases)
        
        assert len(filtered) == 2


class TestDNSRecordGeneration:
    """Tests for DNS record generation functions."""
    
    def test_add_to_unbound(self):
        """Test generating DNS records for adding a lease."""
        core.default_domain = "lan"
        hostname = "test-host"
        address = ipaddress.ip_address("2001:db8::1")
        
        records = core.add_to_unbound(hostname, address)
        
        assert len(records) == 2
        assert "test-host.lan" in records[1]
        assert "IN AAAA 2001:db8::1" in records[1]
        assert "PTR test-host.lan" in records[0]
    
    def test_remove_from_unbound(self):
        """Test generating DNS records for removing a lease."""
        core.default_domain = "lan"
        hostname = "test-host"
        address = ipaddress.ip_address("2001:db8::1")
        
        records = core.remove_from_unbound(hostname, address)
        
        assert len(records) == 2
        assert "test-host.lan" in records[1]
        assert "IN AAAA 2001:db8::1" in records[1]
    
    def test_modify_on_unbound(self):
        """Test generating DNS records for modifying a lease."""
        core.default_domain = "lan"
        hostname = "test-host"
        new_address = ipaddress.ip_address("2001:db8::2")
        old_address = ipaddress.ip_address("2001:db8::1")
        
        add_rr, remove_rr = core.modify_on_unbound(hostname, new_address, old_address)
        
        assert len(add_rr) == 2
        assert len(remove_rr) == 2
        assert "2001:db8::2" in add_rr[1]
        assert "2001:db8::1" in remove_rr[1]


class TestUnboundControl:
    """Tests for unbound-control interactions."""
    
    def test_find_unbound_control_with_which(self, mocker):
        """Test finding unbound-control using which command."""
        core.UNBOUND_CONTROL_PATH = None
        mocker.patch('shutil.which', return_value='/usr/sbin/unbound-control')
        
        path = core.find_unbound_control()
        
        assert path == '/usr/sbin/unbound-control'
    
    def test_find_unbound_control_fallback(self, mocker):
        """Test finding unbound-control using fallback paths."""
        core.UNBOUND_CONTROL_PATH = None
        mocker.patch('shutil.which', return_value=None)
        mocker.patch('os.path.isfile', side_effect=lambda p: p == '/usr/sbin/unbound-control')
        mocker.patch('os.access', return_value=True)
        
        path = core.find_unbound_control()
        
        assert path == '/usr/sbin/unbound-control'
    
    def test_find_unbound_control_not_found(self, mocker):
        """Test exception when unbound-control is not found."""
        core.UNBOUND_CONTROL_PATH = None
        mocker.patch('shutil.which', return_value=None)
        mocker.patch('os.path.isfile', return_value=False)
        
        with pytest.raises(FileNotFoundError):
            core.find_unbound_control()
    
    def test_unbound_control_with_server_and_config(self, mock_unbound_control, tmp_path):
        """Test unbound_control with server and config file parameters."""
        mock_run, mock_which = mock_unbound_control
        
        # Create a temporary config file since code checks if it exists
        config_file = tmp_path / "unbound.conf"
        config_file.write_text("# test config")
        
        core.unbound_control(['status'], server='127.0.0.1:8953', config_file=str(config_file))
        
        mock_run.assert_called_once()
        call_args = mock_run.call_args[0][0]
        assert '-c' in call_args
        assert str(config_file) in call_args
        assert '-s' in call_args
        assert '127.0.0.1:8953' in call_args
        assert 'status' in call_args
    
    def test_apply_unbound_changes_with_additions(self, mock_unbound_control):
        """Test applying DNS record additions to Unbound."""
        mock_run, _ = mock_unbound_control
        
        add_rr = ["test-host.lan IN AAAA 2001:db8::1"]
        core.apply_unbound_changes(True, [], add_rr)
        
        assert mock_run.call_count == 1
        call_args = mock_run.call_args[0][0]
        assert 'local_datas' in call_args
    
    def test_apply_unbound_changes_with_removals(self, mock_unbound_control):
        """Test applying DNS record removals to Unbound."""
        mock_run, _ = mock_unbound_control
        
        remove_rr = ["test-host.lan IN AAAA 2001:db8::1"]
        core.apply_unbound_changes(True, remove_rr, [])
        
        assert mock_run.call_count == 1
        call_args = mock_run.call_args[0][0]
        assert 'local_datas_remove' in call_args


class TestLeaseFileProcessing:
    """Tests for lease file reading and processing."""
    
    def test_read_file_valid(self, sample_lease_file):
        """Test reading a valid lease file."""
        leases = core.read_file(sample_lease_file)
        
        assert len(leases) == 2
        assert leases[0]["hostname"] in ["test-host", "another-host"]
    
    def test_read_file_not_found(self, caplog):
        """Test reading a non-existent file."""
        leases = core.read_file("/nonexistent/file.json")
        
        assert len(leases) == 0
        assert "Error reading file" in caplog.text
    
    def test_read_file_invalid_json(self, temp_lease_dir, caplog):
        """Test reading a file with invalid JSON."""
        invalid_file = os.path.join(temp_lease_dir, "invalid.json")
        with open(invalid_file, 'w') as f:
            f.write("not valid json{")
        
        leases = core.read_file(invalid_file)
        
        assert len(leases) == 0
        assert "Error reading file" in caplog.text
    
    def test_process_lease_file_new_leases(self, sample_lease_file, mock_unbound_control):
        """Test processing a lease file with new leases."""
        core.active_leases = {}
        core.default_domain = "lan"
        mock_run, _ = mock_unbound_control
        
        result = core.process_lease_file(sample_lease_file)
        
        filename = os.path.basename(sample_lease_file)
        assert filename in result
        assert len(result[filename]) == 2
        assert mock_run.call_count == 1
    
    def test_process_lease_file_modified_lease(self, temp_lease_dir, mock_unbound_control):
        """Test processing a lease file with a modified address."""
        core.active_leases = {}
        core.default_domain = "lan"
        mock_run, _ = mock_unbound_control
        
        # Initial lease
        lease_file = os.path.join(temp_lease_dir, "test.json")
        data = [{
            "Address": [0x2001, 0xdb8, 0, 0, 0, 0, 0, 1],
            "AddressType": "IPv6",
            "Hostname": "test-host",
            "Expire": "2025-12-31T23:59:59Z"
        }]
        with open(lease_file, 'w') as f:
            json.dump(data, f)
        
        core.process_lease_file(lease_file)
        mock_run.reset_mock()
        
        # Modified lease
        data[0]["Address"] = [0x2001, 0xdb8, 0, 0, 0, 0, 0, 2]
        with open(lease_file, 'w') as f:
            json.dump(data, f)
        
        core.process_lease_file(lease_file)
        
        # Should have both add and remove operations
        assert mock_run.call_count == 2


class TestInitialRun:
    """Tests for initial run functionality."""
    
    def test_initial_run_single_file(self, sample_lease_file, mock_unbound_control):
        """Test initial run with a single file."""
        core.active_leases = {}
        core.default_domain = "lan"
        
        core.initial_run(sample_lease_file)
        
        assert len(core.active_leases) == 1
    
    def test_initial_run_directory(self, temp_lease_dir, sample_lease_data, mock_unbound_control):
        """Test initial run with a directory."""
        core.active_leases = {}
        core.default_domain = "lan"
        
        # Create multiple lease files
        for i in range(3):
            lease_file = os.path.join(temp_lease_dir, f"lease-{i}.json")
            with open(lease_file, 'w') as f:
                json.dump(sample_lease_data, f)
        
        core.initial_run(temp_lease_dir)
        
        assert len(core.active_leases) == 3
    
    def test_initial_run_nonexistent_source(self, caplog):
        """Test initial run with non-existent source."""
        with pytest.raises(SystemExit):
            core.initial_run("/nonexistent/path")


class TestDirChangeHandler:
    """Tests for directory change handler."""
    
    def test_handler_initialization(self, temp_lease_dir):
        """Test DirChangeHandler initialization."""
        handler = core.DirChangeHandler(temp_lease_dir)
        
        assert handler.source_dir == temp_lease_dir
    
    def test_handler_on_created_event(self, temp_lease_dir, mock_unbound_control, mocker):
        """Test handler response to file creation."""
        core.active_leases = {}
        core.default_domain = "lan"
        
        handler = core.DirChangeHandler(temp_lease_dir)
        
        # Create mock event
        event = Mock()
        event.event_type = "created"
        event.is_directory = False
        event.src_path = os.path.join(temp_lease_dir, "new-lease.json")
        
        # Create the actual file
        data = [{
            "Address": [0x2001, 0xdb8, 0, 0, 0, 0, 0, 1],
            "AddressType": "IPv6",
            "Hostname": "new-host",
            "Expire": "2025-12-31T23:59:59Z"
        }]
        with open(event.src_path, 'w') as f:
            json.dump(data, f)
        
        handler.on_any_event(event)
        
        assert "new-lease.json" in core.active_leases
    
    def test_handler_on_deleted_event(self, sample_lease_file, mock_unbound_control):
        """Test handler response to file deletion."""
        core.active_leases = {}
        core.default_domain = "lan"
        mock_run, _ = mock_unbound_control
        
        # First process the file
        core.process_lease_file(sample_lease_file)
        filename = os.path.basename(sample_lease_file)
        assert filename in core.active_leases
        
        # Mock deletion event
        handler = core.DirChangeHandler(os.path.dirname(sample_lease_file))
        event = Mock()
        event.event_type = "deleted"
        event.src_path = sample_lease_file
        
        handler.on_any_event(event)
        
        assert filename not in core.active_leases


class TestMainFunction:
    """Tests for the main entry point."""
    
    def test_main_with_valid_parameters(self, temp_lease_dir, sample_lease_data, mock_unbound_control, mocker):
        """Test main function with valid parameters."""
        # Create a lease file
        lease_file = os.path.join(temp_lease_dir, "test.json")
        with open(lease_file, 'w') as f:
            json.dump(sample_lease_data, f)
        
        # Mock Observer to avoid blocking
        mock_observer = MagicMock()
        mocker.patch('json_leases_to_unbound.core.Observer', return_value=mock_observer)
        mocker.patch('time.sleep', side_effect=KeyboardInterrupt)
        
        core.active_leases = {}
        
        core.main(
            log_level='INFO',
            source=temp_lease_dir,
            domain='test.local',
            unbound_server=None,
            config_file=None
        )
        
        assert core.default_domain == 'test.local'
        mock_observer.start.assert_called_once()
        mock_observer.stop.assert_called_once()



import pytest
from unittest.mock import patch, MagicMock
from json_leases_to_unbound import cli


class TestCLI:
    """Tests for CLI argument parsing and execution."""
    
    def test_main_cli_default_arguments(self, mocker):
        """Test CLI with default arguments."""
        mock_main = mocker.patch('json_leases_to_unbound.main')
        mock_argv = ['json-leases-to-unbound']
        
        with patch('sys.argv', mock_argv):
            cli.main_cli()
        
        mock_main.assert_called_once()
        call_kwargs = mock_main.call_args[1]
        assert call_kwargs['log_level'] == 'INFO'
        assert call_kwargs['source'] == '/run/slaac-resolver/'
        assert call_kwargs['domain'] == 'lan'
        assert call_kwargs['unbound_server'] is None
        assert call_kwargs['config_file'] is None
    
    def test_main_cli_custom_log_level(self, mocker):
        """Test CLI with custom log level."""
        mock_main = mocker.patch('json_leases_to_unbound.main')
        mock_argv = ['json-leases-to-unbound', '--log-level', 'DEBUG']
        
        with patch('sys.argv', mock_argv):
            cli.main_cli()
        
        call_kwargs = mock_main.call_args[1]
        assert call_kwargs['log_level'] == 'DEBUG'
    
    def test_main_cli_custom_source(self, mocker):
        """Test CLI with custom source directory."""
        mock_main = mocker.patch('json_leases_to_unbound.main')
        mock_argv = ['json-leases-to-unbound', '--source', '/tmp/leases']
        
        with patch('sys.argv', mock_argv):
            cli.main_cli()
        
        call_kwargs = mock_main.call_args[1]
        assert call_kwargs['source'] == '/tmp/leases'
    
    def test_main_cli_custom_domain(self, mocker):
        """Test CLI with custom domain."""
        mock_main = mocker.patch('json_leases_to_unbound.main')
        mock_argv = ['json-leases-to-unbound', '--domain', 'home.local']
        
        with patch('sys.argv', mock_argv):
            cli.main_cli()
        
        call_kwargs = mock_main.call_args[1]
        assert call_kwargs['domain'] == 'home.local'
    
    def test_main_cli_with_unbound_server(self, mocker):
        """Test CLI with unbound server specification."""
        mock_main = mocker.patch('json_leases_to_unbound.main')
        mock_argv = ['json-leases-to-unbound', '--unbound-server', '192.168.1.1:8953']
        
        with patch('sys.argv', mock_argv):
            cli.main_cli()
        
        call_kwargs = mock_main.call_args[1]
        assert call_kwargs['unbound_server'] == '192.168.1.1:8953'
    
    def test_main_cli_with_config_file(self, mocker):
        """Test CLI with config file path."""
        mock_main = mocker.patch('json_leases_to_unbound.main')
        mock_argv = ['json-leases-to-unbound', '--config-file', '/etc/unbound/unbound.conf']
        
        with patch('sys.argv', mock_argv):
            cli.main_cli()
        
        call_kwargs = mock_main.call_args[1]
        assert call_kwargs['config_file'] == '/etc/unbound/unbound.conf'
    
    def test_main_cli_all_arguments(self, mocker):
        """Test CLI with all arguments specified."""
        mock_main = mocker.patch('json_leases_to_unbound.main')
        mock_argv = [
            'json-leases-to-unbound',
            '--log-level', 'WARNING',
            '--source', '/custom/path',
            '--domain', 'custom.local',
            '--unbound-server', '10.0.0.1:8953',
            '--config-file', '/custom/unbound.conf'
        ]
        
        with patch('sys.argv', mock_argv):
            cli.main_cli()
        
        call_kwargs = mock_main.call_args[1]
        assert call_kwargs['log_level'] == 'WARNING'
        assert call_kwargs['source'] == '/custom/path'
        assert call_kwargs['domain'] == 'custom.local'
        assert call_kwargs['unbound_server'] == '10.0.0.1:8953'
        assert call_kwargs['config_file'] == '/custom/unbound.conf'
    
    def test_main_cli_invalid_log_level(self, capsys):
        """Test CLI with invalid log level."""
        mock_argv = ['json-leases-to-unbound', '--log-level', 'INVALID']
        
        with patch('sys.argv', mock_argv):
            with pytest.raises(SystemExit):
                cli.main_cli()
        
        captured = capsys.readouterr()
        assert "invalid choice" in captured.err.lower()
